/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "tile.h"
#include "rpg/entity/entity.h"

#define MAP_WIDTH_MAX 2048
#define MAP_HEIGHT_MAX 2048
#define MAP_LAYERS_MAX 4
#define MAP_ENTITIES_MAX 256

typedef struct _map_t {
  tile_t tiles[MAP_WIDTH_MAX * MAP_HEIGHT_MAX * MAP_LAYERS_MAX];
  uint16_t width;
  uint16_t height;
  uint8_t layers;

  entity_t entities[MAP_ENTITIES_MAX];
  uint8_t entityCount;
} map_t;

/**
 * Initializes a map.
 * 
 * @param map Map to initialize.
 * @param width Width of the map.
 * @param height Height of the map.
 * @param layers Number of layers in the map.
 */
void mapInit(
  map_t *map,
  const uint16_t width,
  const uint16_t height,
  const uint8_t layers
);

/**
 * Updates the map and all entities on it.
 * 
 * @param map Map to update.
 */
void mapUpdate(map_t *map);

/**
 * Adds an entity to the map.
 * 
 * @param map Map to add the entity to.
 * @param entity Entity to add to the map.
 */
entity_t * mapEntityAdd(map_t *map);

/**
 * Gets the entity at the specified position.
 * 
 * @param map Map to get the entity from.
 * @param x X position of the entity.
 * @param y Y position of the entity.
 * @return Entity at the specified position, or NULL if no entity is there.
 */
entity_t * mapEntityGetByPosition(
  map_t *map,
  const uint16_t x,
  const uint16_t y
);

/**
 * Gets the tile at the specified position.
 * 
 * @param map Map to get the tile from.
 * @param x X position of the tile.
 * @param y Y position of the tile.
 * @param layer Layer of the tile.
 * @return Tile at the specified position.
 */
tile_t mapTileGetByPosition(
  const map_t *map,
  const uint16_t x,
  const uint16_t y,
  const uint8_t layer
);

/**
 * Quickly set an entire layer of tiles at once.
 * 
 * @param map Map to set the tiles on.
 * @param ids Array of tiles to set.
 * @param layer Layer to set the tiles on.
 */
void mapTilesSet(
  map_t *map,
  const tile_t tiles[],
  const uint8_t layer
);